
//// UN-TWEAKABLES - AUTOMATICALLY-TRACKED TRANSFORMS ////////////////

float4x4 WorldITXf : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WvpXf : WorldViewProjection < string UIWidget="None"; >;
float4x4 WorldXf : World < string UIWidget="None"; >;
float4x4 ViewIXf : ViewInverse < string UIWidget="None"; >;

//// TWEAKABLE PARAMETERS ////////////////////

float etaRatioR <
	  string UIWidget = "slider";
    float UIMin = 0;
    float UIMax = 1;
	  float  UIStep = 0.01;
    string UIName = "Eta Red";
> = 0.8;

float etaRatioG <
	  string UIWidget = "slider";
    float  UIMin = 0;
    float  UIMax = 1;
	  float  UIStep = 0.01;
    string UIName = "Eta Green";
> = 0.85;

float etaRatioB <
	  string UIWidget = "slider";
    float  UIMin = 0;
    float  UIMax = 1;
	  float  UIStep = 0.01;
    string UIName = "Eta Blue";
> = 0.9;

float fresnelPower <
	  string UIWidget = "slider";
    float  UIMin = 0;
    float  UIMax = 10;
	  float  UIStep = 0.1;
    string UIName = "FresnelPower";
> = 2;

float fresnelScale <
	  string UIWidget = "slider";
    float  UIMin   = 0;
    float  UIMax   = 1.0;
	  float  UIStep  = 0.01;
    string UIName = "Fresnel Scale";
> = 1.0;

float fresnelBias <
	  string UIWidget = "slider";
    float  UIMin = 0;
    float  UIMax = 0.2;
	  float  UIStep = 0.01;
    string UIName = "Fresnel Bias";
> = 0.1;


//////// COLOR & TEXTURE /////////////////////

texture NormalTexture : NORMAL <
    string ResourceName = "Earth_TSNormal.dds";
    string UIName =  "Tangent Space Normal Texture";
    string ResourceType = "2D";
>;
sampler2D NormalMapSampler = sampler_state {
    Texture = <NormalTexture>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};  

texture EnvMap : ENVIRONMENT <
    string ResourceName = "EnvironmentMapMS.dds";
    string UIName =  "Environment";
    string ResourceType = "Cube";
>;

samplerCUBE EnvMapSampler = sampler_state 
{
    texture = <EnvMap>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU  = Clamp;        
    AddressV  = Clamp;
    AddressW  = Clamp;
};

// shared shadow mapping supported in Cg version

//////// CONNECTOR DATA STRUCTURES ///////////

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
    float4 Tangent	: TANGENT0;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float2 UV		    : TEXCOORD0;
    // The following values are passed in "World" coordinates since
    //   it tends to be the most flexible and easy for handling
    //   reflections, sky lighting, and other "global" effects.
    float3 WorldNormal	 : TEXCOORD2;
    float3 WorldTangent	 : TEXCOORD3;
    float3 WorldBinormal : TEXCOORD4;
    float3 WorldView	 : TEXCOORD5;
};
 
///////// VERTEX SHADING /////////////////////

/*********** Generic Vertex Shader ******/

vertexOutput std_VS(appdata IN) {
    vertexOutput OUT = (vertexOutput)0;
	
    OUT.WorldNormal   = mul(IN.Normal,WorldITXf).xyz;
    OUT.WorldTangent  = mul(IN.Tangent,WorldITXf).xyz;
  	// Binormale in World Space
	OUT.WorldBinormal = cross(OUT.WorldNormal,OUT.WorldTangent).xyz;

    float4 Position = float4(IN.Position.xyz,1);
    float3 PosWorld = mul(Position,WorldXf).xyz;
	
	OUT.UV = IN.UV;
	
    OUT.WorldView = normalize(ViewIXf[3].xyz - PosWorld);
    OUT.HPosition = mul(Position,WvpXf);
    return OUT;
}

///////// PIXEL SHADING //////////////////////

float4 reflect_PS(vertexOutput IN) : COLOR {
	
   	// compute matrix for tangent space transformation 
	float3x3 matWorldToTangentSpace;
	matWorldToTangentSpace[0] = normalize(IN.WorldTangent);
	matWorldToTangentSpace[1] = normalize(IN.WorldBinormal);
	matWorldToTangentSpace[2] = normalize(IN.WorldNormal);
	
    float3 normalMap = tex2D(NormalMapSampler, IN.UV).xyz* 2.0 - 1.0;
	 
	// compute parameters N,L,V,H 
    float3 Nn = normalize(mul(normalMap,matWorldToTangentSpace));
    float3 Vn = normalize(IN.WorldView);
	float3 Rn = -reflect(Vn,Nn);
	
	float3 envColor = texCUBE(EnvMapSampler, Rn);
	
    float3 result = envColor;
	
    return float4(result,1);
}

float4 refract_PS(vertexOutput IN) : COLOR {
	
   	// compute matrix for tangent space transformation 
	float3x3 matWorldToTangentSpace;
	matWorldToTangentSpace[0] = normalize(IN.WorldTangent);
	matWorldToTangentSpace[1] = normalize(IN.WorldBinormal);
	matWorldToTangentSpace[2] = normalize(IN.WorldNormal);
	
    float3 normalMap = tex2D(NormalMapSampler, IN.UV).xyz* 2.0 - 1.0;
	 
	// compute parameters N,L,V,H 
    float3 Nn = normalize(mul(normalMap,matWorldToTangentSpace));
    float3 Vn = normalize(IN.WorldView);
	float3 Rn = -reflect(Vn,Nn);
	float3 Tn = refract(Vn,Nn, etaRatioR);
	
	float3 reflectedColor = texCUBE(EnvMapSampler, Rn);
	float3 refractedColor = texCUBE(EnvMapSampler, Tn);
	
    float fresnel = fresnelBias + fresnelScale * pow(1.0 - dot(Nn,Vn), fresnelPower);
   
	
    float3 result = lerp(refractedColor, reflectedColor, fresnel);
	
    return float4(result,1);
}

float4 chromatic_PS(vertexOutput IN) : COLOR {
	
     	// compute matrix for tangent space transformation 
	float3x3 matWorldToTangentSpace;
	matWorldToTangentSpace[0] = normalize(IN.WorldTangent);
	matWorldToTangentSpace[1] = normalize(IN.WorldBinormal);
	matWorldToTangentSpace[2] = normalize(IN.WorldNormal);
	
    float3 normalMap = tex2D(NormalMapSampler, IN.UV).xyz* 2.0 - 1.0;
	 
	// compute parameters N,L,V,H 
    float3 Nn = normalize(mul(normalMap,matWorldToTangentSpace));
    float3 Vn = normalize(IN.WorldView);
	float3 Rn = -reflect(Vn,Nn);
	float3 TnR = refract(Vn,Nn, etaRatioR);
	float3 TnG = refract(Vn,Nn, etaRatioG);
	float3 TnB = refract(Vn,Nn, etaRatioB);
	
	float3 reflectedColor = texCUBE(EnvMapSampler, Rn);
	float3 refractedColor = (float3)0;
	refractedColor.r = texCUBE(EnvMapSampler, TnR).r;
	refractedColor.g = texCUBE(EnvMapSampler, TnG).g;
	refractedColor.b = texCUBE(EnvMapSampler, TnB).b;
	
    float fresnel = fresnelBias + fresnelScale * pow(1.0 - dot(Nn,Vn), fresnelPower);
   
	
    float3 result = lerp(refractedColor, reflectedColor, fresnel);
	
    return float4(result,1);

}
///// TECHNIQUES /////////////////////////////

RasterizerState DisableCulling
{
    CullMode = NONE;
};

DepthStencilState DepthEnabling
{
	DepthEnable = TRUE;
};

BlendState DisableBlend
{
	BlendEnable[0] = FALSE;
};

technique10 Reflect10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, reflect_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}
technique10 Refract10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, refract_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}

technique10 Chromatic10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, chromatic_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}

/////////////////////////////////////// eof //
